/*
News shock model

E. Sims "News, Non-Invertibility, and Structural VARs" (2012)
Slightly modified to potentially include rule-of-thumb price-setters (Gali & Gertler, 1999)
Note: All variables below are in logs (the variable "i" is log of the gross nominal interest rate)

MPM, 2015-07-25
*/

var lambda_hat mu_hat y_hat c_hat I_hat adj_cost k_hat n w_hat R mc p_rel pi v A B i i_minus_pi i_real a_hat;
varexo a_shock news_shock i_shock;
parameters beta delta g_a pi_star alpha xi gamma tau phi omega epsilon rho psi_pi psi_y n_ss
           Delta R_ss i_ss k_hat_ss y_hat_ss I_hat_ss c_hat_ss w_hat_ss lambda_hat_ss theta p_rel_ss v_ss B_ss A_ss mc_ss;

// Set parameters
set_param_value('beta', beta);
set_param_value('delta', delta);
set_param_value('g_a', g_a);
set_param_value('pi_star', pi_star);
set_param_value('alpha', alpha);
set_param_value('xi', xi);
set_param_value('gamma', gamma);
set_param_value('tau', tau);
set_param_value('phi', phi);
set_param_value('omega', omega);
set_param_value('epsilon', epsilon);
set_param_value('rho', rho);
set_param_value('psi_pi', psi_pi);
set_param_value('psi_y', psi_y);
set_param_value('n_ss', n_ss);
set_param_value('Delta', Delta);
set_param_value('mc_ss', mc_ss);
set_param_value('R_ss', R_ss);
set_param_value('i_ss', i_ss);
set_param_value('k_hat_ss', k_hat_ss);
set_param_value('y_hat_ss', y_hat_ss);
set_param_value('I_hat_ss', I_hat_ss);
set_param_value('c_hat_ss', c_hat_ss);
set_param_value('w_hat_ss', w_hat_ss);
set_param_value('lambda_hat_ss', lambda_hat_ss);
set_param_value('theta', theta);
set_param_value('p_rel_ss', p_rel_ss);
set_param_value('v_ss', v_ss);
set_param_value('B_ss', B_ss);
set_param_value('A_ss', A_ss);
set_param_value('mc_ss', mc_ss);

// Model equations
model;

    //TFP growth
    a_hat = g_a + a_shock + news_shock(-3); // Set anticipation horizon here

    //Labor factor demand
    exp(w_hat + alpha/(1-alpha)*a_hat) = (1-alpha)*exp(mc + alpha*(k_hat(-1) - n));

    //Capital factor demand
    exp(R) = alpha*exp(a_hat + mc + (alpha-1)*(k_hat(-1) - n));

    //Consumption Euler equation
    exp(lambda_hat) = 1/(exp(c_hat) - gamma*exp(c_hat(-1) - a_hat/(1-alpha))) - beta*gamma/(exp(c_hat(+1) + a_hat(+1)/(1-alpha)) - gamma*exp(c_hat));

    //Consumption-leisure tradeoff
    theta*exp(xi*n) = exp(lambda_hat + w_hat);

    //Bond Euler equation
    exp(lambda_hat) = beta*exp(lambda_hat(+1) + i - pi(+1) - a_hat(+1)/(1-alpha));

    //Capital Euler equation
    exp(mu_hat) = beta*exp(-a_hat(+1)/(1-alpha))*(exp(lambda_hat(+1) + R(+1)) + (1-delta)*exp(mu_hat(+1)));

    //Term entering into adjustment costs
    adj_cost = exp(I_hat - I_hat(-1) + a_hat/(1-alpha)) - Delta;

    //Investment "Q" equation
    exp(lambda_hat) = exp(mu_hat)*(1 - tau/2*(adj_cost)^2 - tau*adj_cost*(adj_cost + Delta)) + beta*tau*(exp(mu_hat(+1) - a_hat(+1)/(1-alpha))*adj_cost(+1)*(adj_cost(+1) + Delta)^2);

    //Evolution of capital stock
    exp(k_hat) = (1 - tau/2*adj_cost^2)*exp(I_hat) + (1-delta)*exp(k_hat(-1) - a_hat/(1-alpha));

    //Aggregate resource constraint
    exp(y_hat) = exp(c_hat) + exp(I_hat);

    //Aggregate production function
    exp(y_hat) = exp(alpha/(1-alpha)*a_hat + alpha*k_hat(-1) + (1-alpha)*n - v);

    //Evolution of price dispersion
    exp(v) = (1-phi)*((1-omega)*exp(-epsilon*p_rel) + omega*exp(epsilon*(pi - pi(-1)))) + phi*exp(epsilon*pi + v(-1));

    //Relationship between relative reset price and inflation gap
    1 = (1-phi)*((1-omega)*exp((1-epsilon)*p_rel) + omega*exp((epsilon-1)*(pi - pi(-1)))) + phi*exp((epsilon-1)*pi);

    //Optimal price reset
    1 = exp(B + p_rel - A);

    //Numerator in optimal price reset formula
    exp(A) = epsilon/(epsilon-1)*exp(lambda_hat + y_hat + mc) + phi*beta*exp(epsilon*pi(+1) + A(+1));

    //Denominator in optimal price reset formula
    exp(B) = exp(lambda_hat + y_hat) + phi*beta*exp((epsilon-1)*pi(+1) + B(+1));

    //Nominal interest rate rule
    i - pi_star = rho*(i(-1) - pi_star) + (1-rho)*(log(Delta/beta) + psi_pi*(exp(pi - pi_star) - 1) + psi_y*(exp(y_hat - y_hat(-1) + a_hat/(1-alpha)) - Delta)) + i_shock;

    //Nominal interest rate minus inflation
    i_minus_pi = i - pi;

    //Ex-ante real interest rate
    i_real = i - pi(+1);

end;

// Steady states
initval;
    lambda_hat = lambda_hat_ss;
    mu_hat = lambda_hat_ss;
    y_hat = y_hat_ss;
    c_hat = c_hat_ss;
    I_hat = I_hat_ss;
    adj_cost = 0;
    k_hat = k_hat_ss;
    n = n_ss;
    w_hat = w_hat_ss;
    R = R_ss;
    mc = mc_ss;
    p_rel = p_rel_ss;
    pi = pi_star;
    v = v_ss;
    A = A_ss;
    B = B_ss;
    a_hat = g_a;
    i = i_ss;
    i_minus_pi = i_ss - pi_star;
    i_real = i_ss - pi_star;
end;

steady(maxit=0);
check;

// Shock settings
shocks;
    var a_shock = 1;
    var news_shock = 1;
    var i_shock = 1;
end;

stoch_simul(irf=40, order=1);
