function [samples, epsilon, avg_acceptprob, L] = nuts_da_svma(f, nuts, theta0)

% NUTS algorithm
% Copyright (c) 2011, Matthew D. Hoffman
% http://www.cs.princeton.edu/~mdhoffma/
%
% Modified by Mikkel Plagborg-Moller, May 2015
% All modifications are marked with comment "MPM modification"

% Arguments:
% f                 fct                 log target density function, able to supply gradient as second output
% nuts              struct              Matlab structure with NUTS settings
% theta0            1 x D               initial draw

% Outputs:
% samples           M/store_every x D   stored draws after adaptation
% epsilon           1 x 1               adapted step size
% avg_acceptprob    M/store_every x 1   avg. NUTS acceptance rate after adaptation
% L                 1 x D               square root of mass matrix


% The following is the original introduction to the function as written by Matthew D. Hoffman:

% 
% Implements the No-U-Turn Sampler (NUTS), specifically, algorithm 6
% from the NUTS paper (Hoffman & Gelman, 2011). Runs Madapt steps of
% burn-in, during which it adapts the step size parameter epsilon, then
% starts generating samples to return.
% 
% epsilon is a step size parameter.
% f(theta) should be a function that returns the log probability its
% gradient evaluated at theta. I.e., you should be able to call 
% [logp grad] = f(theta).
% M is the number of samples to generate.
% Madapt is the number of steps of burn-in/how long to run the dual
% averaging algorithm to fit the step size epsilon. 
% theta0 is a 1-by-D vector with the desired initial setting of the parameters.
% delta should be between 0 and 1, and is a target HMC acceptance
% probability. Defaults to 0.6 if unspecified.
% 
% The returned variable "samples" is an M-by-D matrix of post-burn-in samples 
% generated by NUTS.
% The returned variable "epsilon" is the step size that was fit using dual
% averaging.

% Copyright (c) 2011, Matthew D. Hoffman
% All rights reserved.
% 
% Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
% 
% Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
% Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

global nfevals;
nfevals = 0;

% Begin MPM modification
M = nuts.num; % Number of MCMC steps to take after adaptation
Madapt = nuts.adapt; % Number of steps to take in adaptation
store_every = nuts.store_every; % Store current draw for every X-th step
delta = nuts.target_rate; % Target acceptance rate
mass_adapt = nuts.mass_adapt; % =1: Adapt to posterior std. dev. of parameters
mass_adapt_times = nuts.mass_adapt_times; % Times between which to adapt. First entry is when adaptation starts. At second entry, the first std. dev. estimate is produced. At third entry, second estimate is produced. Adaptation stops at last entry.
mass_adapt_shrinkage = nuts.mass_adapt_shrinkage; % Shrinkage constant for adaptation, higher values mean estimated std. dev's are less variable between parameters
min_epsilon = nuts.min_epsilon; % Minimum allowed step size
jitter = nuts.jitter; % Step size jitter factor (after adaptation), btw 0 and 1 (0 = no jitter, 1 = jitter from 0% to 200%)
max_depth = nuts.max_depth; % Max NUTS tree depth
print_epsilon = nuts.print_epsilon; % =1: Display step size in each NUTS iteration
% End MPM modification

assert(size(theta0, 1) == 1);

D = length(theta0);

samples = zeros(floor((M+Madapt)/store_every), D); % MPM modification: if desired, stores fewer samples than total number of steps
avg_acceptprob = NaN(M+Madapt,1); % MPM modification: Will contain avg. NUTS acceptance probability in each MCMC step

[logp, grad] = f(theta0);

% Begin MPM modification
current_sample = theta0; % Keep track of current sample (may not store all samples)
L = 1; % Initialize square root of mass matrix

if mass_adapt == 1; % If adaptation of mass matrix should take place, initialize Welford accumulation
    k = 0;
    sample_M = zeros(1, D);
    sample_V = zeros(1, D);
end;

% Initialize Hamiltonian
hamiltonian = struct;
hamiltonian.x_L = @(x) x; % Will return  x*L
hamiltonian.x_Lp = @(x) x; % Will return x*L'
hamiltonian.Linv_xp = @(x) x'; % Will return L^{-1}*x'
% End MPM modification

% Choose a reasonable first epsilon by a simple heuristic.
epsilon = find_reasonable_epsilon(theta0, grad, logp, f, hamiltonian); % MPM modification: extra "hamiltonian" argument
fprintf('%-20s%6.4e\n', 'Initial epsilon:', epsilon); % MPM modification: display initial step size

% Begin MPM modification
if Madapt == 0;
    epsilon = min_epsilon;
end;
% End MPM modification

% Parameters to the dual averaging algorithm.
gamma = 0.05;
t0 = 10;
kappa = 0.75;
mu = log(10*epsilon);
% Initialize dual averaging algorithm.
r = 0; % MPM modification: steps since beginning of current adaptation
epsilonbar = 1;
Hbar = 0;

for m = 2:M+Madapt;
    
    % Resample momenta.
    r0 = randn(1, D);
    % Joint log-probability of theta and momenta r.
    joint = logp - 0.5 * (r0 * r0');
    % Resample u ~ uniform([0, exp(joint)]).
    % Equivalent to (log(u) - joint) ~ exponential(1).
    logu = joint - exprnd(1);
    % Initialize tree.
    thetaminus = current_sample; % MPM modification: current sample instead of last stored sample
    thetaplus = current_sample; % MPM modification: current sample instead of last stored sample
    rminus = r0;
    rplus = r0;
    gradminus = grad;
    gradplus = grad;
    % Initial height j = 0.
    j = 0;
    % If all else fails, the next sample is the previous sample.
    next_sample = current_sample; % MPM modification: current sample instead of last stored sample
    % Initially the only valid point is the initial point.
    n = 1;

    % Main loop---keep going until the criterion s == 0.
    s = 1;
    while (s == 1)
        % Choose a direction. -1=backwards, 1=forwards.
        v = 2*(rand() < 0.5)-1;
        % Double the size of the tree.
        if (v == -1)
            [thetaminus, rminus, gradminus, ~, ~, ~, thetaprime, gradprime, proposal_logp, nprime, sprime, alpha, nalpha] = ...
                build_tree(thetaminus, rminus, gradminus, logu, v, j, epsilon, f, joint, hamiltonian); % MPM modification: extra "hamiltonian" argument
        else
            [~, ~, ~, thetaplus, rplus, gradplus, thetaprime, gradprime, proposal_logp, nprime, sprime, alpha, nalpha] = ...
                build_tree(thetaplus, rplus, gradplus, logu, v, j, epsilon, f, joint, hamiltonian); % MPM modification: extra "hamiltonian" argument
        end
        % Use Metropolis-Hastings to decide whether or not to move to a
        % point from the half-tree we just generated.
        if ((sprime == 1) && (rand() < nprime/n))
            next_sample = thetaprime; % MPM modification: current sample instead of last stored sample
            logp = proposal_logp;
            grad = gradprime;
        end
        % Update number of valid points we've seen.
        n = n + nprime;
        % Decide if it's time to stop.
        s = sprime && stop_criterion(thetaminus, thetaplus, rminus, rplus, hamiltonian) && (j<max_depth); % MPM modification: Added condition on max tree depth, extra "hamiltonian" argument

        % Increment depth.
        j = j + 1;

        % Begin MPM modification
        if j==max_depth; % Show warning if max tree depth has been reached
            warning('Max tree depth reached.');
        end;
        % End MPM modification

    end

    % Do adaptation of epsilon if we're still doing burn-in.
    if m <= Madapt;
        r = r+1; % MPM modification: increment r
        eta = 1 / (r + t0); % MPM modification: r instead of m-1
        Hbar = (1 - eta) * Hbar + eta * (delta - alpha / nalpha);
        epsilon = max(exp(mu - sqrt(r)/gamma * Hbar), min_epsilon); % MPM modification: enforce minimum stepsize
        eta = r^-kappa; % MPM modification: r instead of m-1
        epsilonbar = exp((1 - eta) * log(epsilonbar) + eta * log(epsilon));

        if m == Madapt;
            fprintf('%-20s%6.4e\n', 'Final epsilon:', epsilonbar); % MPM modification: output final adapted stepsize
        end;
    end;
    
    if m >= Madapt;
        epsilon = epsilonbar * (1 + jitter*(2*rand - 1)); % MPM modification: step size jitter
    end

    % Begin MPM modification
    if print_epsilon == 1;
        fprintf('%.4e\n', epsilon);
    end;

    avg_acceptprob(m) = alpha/nalpha; % Avg. acceptance rate in current NUTS step
    
    current_sample = next_sample; % Move to new sample
    if mod(m,store_every) == 0; % For every "store_every" draws...
        samples(m/store_every,:) = current_sample; % Store current sample
    end;
    
    if mass_adapt == 1 && m <= mass_adapt_times(end); % If mass matrix should be adapted and we're within the adaptation period...
        
        % Welford accumulation for accurately calculating sample variances over current adaptation window
        k = k+1;
        contrib = current_sample - sample_M;
        sample_M = sample_M + contrib/k; % Update sample mean
        sample_V = sample_V + (current_sample - sample_M).*contrib; % Update sample variances
        
        this_ind = find(mass_adapt_times == m);
        if this_ind >= 1; % If current iteration is the start or end of an adaptation window...

            if this_ind > 1; % If it is the end of a window, update mass matrix
                
                sample_var = sample_V/(k-1); % Sample variances/covariance
                
                L = sqrt(k/(k+5)*sample_var + mass_adapt_shrinkage*5/(k+5)*ones(1,D)); % Square root of new diagonal mass matrix
                L = L/max(L); % Normalize
                
                % Update Hamiltonian
                hamiltonian.x_L = @(x) x.*L;
                hamiltonian.x_Lp = @(x) x.*L;
                hamiltonian.Linv_xp = @(x) (x./L)';
                disp(L); % Display mass vector
                
                % Reset epsilon adaptation variables
                epsilon = epsilonbar;
                mu = log(10*epsilonbar);
                r = 0;
                Hbar = 0;
                epsilonbar = 1;
                
            end;

            % Reset Welford accumulation
            sample_M = zeros(1, D);
            sample_V = zeros(size(sample_V));
            k = 0;

        end;
    
    end;
    
    if mod(m, floor((M+Madapt)/100)) == 0;
              fprintf('%6d%10.1f%1s%30s%4.3f\n', m, 100*m/(M+Madapt), '%', 'Accept. freq. last 50: ', nanmean(avg_acceptprob(max(1,m-49):m))); % Print out progress and avg. acceptance rate in last 50 steps
    end;
    % End MPM modification
end
samples = samples(floor(Madapt/store_every)+1:end, :); % MPM modification: return stored samples after adaptation
avg_acceptprob = avg_acceptprob(Madapt+1:end); % MPM modification: return avg. acceptance rate after adaptation
fprintf('Took %d gradient evaluations.\n', nfevals);
end

function [thetaprime, rprime, gradprime, logpprime] = leapfrog(theta, r, grad, epsilon, f, hamiltonian) % MPM modification: extra "hamiltonian" argument
rprime = r + 0.5 * epsilon * hamiltonian.x_Lp(grad); % MPM modification: mass matrix from hamiltonian
thetaprime = theta + epsilon * hamiltonian.x_L(rprime); % MPM modification: mass matrix from hamiltonian
[logpprime, gradprime] = f(thetaprime);
rprime = rprime + 0.5 * epsilon * hamiltonian.x_Lp(gradprime); % MPM modification: mass matrix from hamiltonian
global nfevals;
nfevals = nfevals + 1;
end

function criterion = stop_criterion(thetaminus, thetaplus, rminus, rplus, hamiltonian) % MPM modification: extra "hamiltonian" argument
thetavec = thetaplus - thetaminus;
criterion = (thetavec * hamiltonian.Linv_xp(rminus) >= 0) && (thetavec * hamiltonian.Linv_xp(rplus) >= 0); % MPM modification: mass matrix from hamiltonian
end

% The main recursion.
function [thetaminus, rminus, gradminus, thetaplus, rplus, gradplus, thetaprime, gradprime, logpprime, nprime, sprime, alphaprime, nalphaprime] = ...
                build_tree(theta, r, grad, logu, v, j, epsilon, f, joint0, hamiltonian) % MPM modification: extra "hamiltonian" argument
if (j == 0)
    % Base case: Take a single leapfrog step in the direction v.
    [thetaprime, rprime, gradprime, logpprime] = leapfrog(theta, r, grad, v*epsilon, f, hamiltonian); % MPM modification: extra "hamiltonian" argument
    joint = logpprime - 0.5 * (rprime * rprime');
    % Is the new point in the slice?
    nprime = logu < joint;
    % Is the simulation wildly inaccurate?
    sprime = logu - 1000 < joint;
    % Set the return values---minus=plus for all things here, since the
    % "tree" is of depth 0.
    thetaminus = thetaprime;
    thetaplus = thetaprime;
    rminus = rprime;
    rplus = rprime;
    gradminus = gradprime;
    gradplus = gradprime;
    % Compute the acceptance probability.
    alphaprime = min(1, exp(logpprime - 0.5 * (rprime * rprime') - joint0));
    nalphaprime = 1;
else
    % Recursion: Implicitly build the height j-1 left and right subtrees.
    [thetaminus, rminus, gradminus, thetaplus, rplus, gradplus, thetaprime, gradprime, logpprime, nprime, sprime, alphaprime, nalphaprime] = ...
                build_tree(theta, r, grad, logu, v, j-1, epsilon, f, joint0, hamiltonian); % MPM modification: extra "hamiltonian" argument
    % No need to keep going if the stopping criteria were met in the first
    % subtree.
    if (sprime == 1)
        if (v == -1)
            [thetaminus, rminus, gradminus, ~, ~, ~, thetaprime2, gradprime2, logpprime2, nprime2, sprime2, alphaprime2, nalphaprime2] = ...
                build_tree(thetaminus, rminus, gradminus, logu, v, j-1, epsilon, f, joint0, hamiltonian); % MPM modification: extra "hamiltonian" argument
        else
            [~, ~, ~, thetaplus, rplus, gradplus, thetaprime2, gradprime2, logpprime2, nprime2, sprime2, alphaprime2, nalphaprime2] = ...
                build_tree(thetaplus, rplus, gradplus, logu, v, j-1, epsilon, f, joint0, hamiltonian); % MPM modification: extra "hamiltonian" argument
        end
        % Choose which subtree to propagate a sample up from.
        if (rand() < nprime2 / (nprime + nprime2))
            thetaprime = thetaprime2;
            gradprime = gradprime2;
            logpprime = logpprime2;
        end
        % Update the number of valid points.
        nprime = nprime + nprime2;
        % Update the stopping criterion.
        sprime = sprime && sprime2 && stop_criterion(thetaminus, thetaplus, rminus, rplus, hamiltonian); % MPM modification: extra "hamiltonian" argument
        % Update the acceptance probability statistics.
        alphaprime = alphaprime + alphaprime2;
        nalphaprime = nalphaprime + nalphaprime2;
    end
end
end

function epsilon = find_reasonable_epsilon(theta0, grad0, logp0, f, hamiltonian) % MPM modification: extra "hamiltonian" argument
r0 = randn(1, length(theta0));
epsilon = 1;
% Figure out what direction we should be moving epsilon.

% Begin MPM modification
w = warning; % Save warning settings
warning('error', 'MATLAB:illConditionedMatrix'); % Make warnings about ill-conditioned matrices into errors
warning('error', 'MATLAB:nearlySingularMatrix');
no_error = false;
while ~no_error; % Until a leapfrog step can be taken without error...
    try
        [~, rprime, ~, logpprime] = leapfrog(theta0, r0, grad0, epsilon, f, hamiltonian); % Try leapfrog step with current epsilon...
        no_error = true; % If it goes right, stop loop
    catch err % If there is an error...
        if strcmp(err.identifier, 'MATLAB:illConditionedMatrix') || strcmp(err.identifier, 'MATLAB:nearlySingularMatrix'); % If the error is due to ill-conditioned matrix...
            epsilon = epsilon/10; % Move to smaller epsilon
            fprintf('%-30s%.1e\n', 'Trying smaller epsilon:', epsilon);
        else % Otherwise, throw error as usual
            err.rethrow;
        end;
    end;
end;
warning(w); % Reset warnings to standard setting
%End MPM modification

acceptprob = exp(logpprime - logp0 - 0.5 * (rprime * rprime' - r0 * r0'));
a = 2 * (acceptprob > 0.5) - 1;
% Keep moving epsilon in that direction until acceptprob crosses 0.5.
while (acceptprob^a > 2^(-a))
    epsilon = epsilon * 2^a;
    [~, rprime, ~, logpprime] = leapfrog(theta0, r0, grad0, epsilon, f, hamiltonian); % MPM modification: extra "hamiltonian" argument
    acceptprob = exp(logpprime - logp0 - 0.5 * (rprime * rprime' - r0 * r0'));
end
end









