# delimit ;
clear all;
cap log close;
cap file close _all;
cap program drop _all;
set scrollbufsize 1000000;
set more off;
log using create.log, replace;

*************************************************************;
*   MPM, 2015-07-14;
*   Generates database with economic data;
************************************************************;


*** Settings ***;

* Decide what to do;
local load_data = 1; // Load and save raw data;
local create_db = 1; // Create database with final data series;
local export_csv = 1; // Export CSV with final data;

* File names;
local fred_files = "news_shocks_Monthly.txt news_shocks_Quarterly.txt"; //St. Louis FRED files to load;
local fernald_file = "quarterly_tfp.xls"; //Fernald (2014) file name;

* Sample;
local sample = "1954q3,2007q4"; // Use only data from this sample when creating final database;
local sample_export = "1954q4,2007q4"; // CSV export sample;
* Export variables;
local vars_export = "D_TFP D_RGDPpercap rir_nfb";

*** End settings ***;


if `load_data' == 1 {; //If raw data should be loaded and saved...;

	* Start databases;
	foreach freq in m q {; // For each frequency...;
	
		clear all;
		gen time = .; // Generate time variable;
		save data_`freq', replace; // Save empty database;
		
	};
	
	* FRED;
	foreach f of local fred_files {; // For each FRED file...;
		
		import delimited using ../data/fred/`f', clear delim("\t") varnames(1); // Import TSV;
		
		gen the_date = date(date, "YMD");
		
		if `=`=the_date[2]' - `=the_date[1]'' > 35 {;
			local freq = "q";
		};
		else {;
			local freq = "m";
		};
		
		gen time = `freq'ofd(the_date); // Time variable;
		drop date the_date; // Drop date string;
		merge 1:1 time using data_`freq', nogenerate update; // Merge with previous data;
		save data_`freq', replace;
		
	};
	
	* Fernald (2014);
	import excel using ../data/fernald/`fernald_file', clear sheet("quarterly") firstrow cellrange(A2); // Import from excel;
	gen time = quarterly(date, "YQ"); // Time variable;
	keep time dtfp; // Keep time, TFP and utilization-adjusted TFP;
	keep if !missing(time); // Drop rows that don't have actual data;
	merge 1:1 time using data_q, nogenerate update; // Merge with previous data;
	save data_q, replace;
	
	* Format databases;
	foreach freq in m q {; // For each frequency...;
	
		use data_`freq', clear; //Open database;
		order time; // Order time variable first;
		format time %t`freq'; // Format time variable;
		tsset time;
		compress;
		save data_`freq', replace;
		
	};

};


if `create_db' == 1 {; // If database with final data series should be produced...;

	* Convert monthly to quarterly;
	use data_m, clear; // Open monthly database;
	gen q = qofd(dofm(time)); // Quarter identifier for each month;
	tsset, clear;
	collapse (mean) fedfunds, by(q); // Collapse to quarterly averages;
	rename q time; // New quarterly time variable;
	merge 1:1 time using data_q, nogenerate update; // Merge with other quarterly data;
	
	format time %tq;
	tsset time;
	
	keep if tin(`sample'); // Keep only relevant sample;
	
	gen D_TFP = dtfp/400; // Log quarterly change in TFP;
	
	gen log_RGDPpercap = log(a939rx0q048sbea); // Log real GDP per capita (BEA series, i.e., total population);
	gen D_RGDPpercap = D.log_RGDPpercap; // Log quarterly change;
	
	gen log_PGDP_nfb = log(ipdnbs); // Log GDP deflator, nonfarm business sector;
	gen infl_nfb = D.log_PGDP_nfb; // GDP deflator inflation, nonfarm business sector;
	gen rir_nfb = fedfunds/400 - infl_nfb; // "Real interest rate": effective FFR minus (same-period) inflation;
	
	compress;
	save data_q_final, replace;

};


if `export_csv' == 1 {; // If data should be exported to CSV...;

	use data_q_final, clear;
	gen export_time = mofd(dofq(time));
	format export_time %tmCCYY-NN;
	
	export delimited export_time `vars_export' using data_q_final.csv if tin(`sample_export'), delim(",") replace;

};


log close;

